@@ -29,6 +29,8 @@ import (
 	term "golang.org/x/term"
 )
 
+var version string // Patched by -X option in Makefile
+
 // Define a couplee of partial capability tables for querying
 // checkout directories.
 
@@ -805,6 +807,8 @@ func checkout(outdir string, rev string) string {
 		return outdir
 	} else if vcs.name == "bzr" {
 		croak("checkout is not yet supported in bzr.")
+	} else if vcs.name == "brz" {
+		croak("checkout is not yet supported in bzr.")
 	} else if vcs.name == "hg" {
 		spec := ""
 		if rev != "" {
