@@ -16,8 +16,8 @@ import (
 
 // Most knowledge about specific version-control systems lives in the
 // following class list. Exception; there's a git-specific hook in the
-// repo reader; also see the extractor classes; also see the dump method
-// in the Blob() class.
+// repo reader; also see the extractor classes; also see the branch rename
+// implementation (has amn svn special case).
 //
 // Import/export style flags are as follows:
 //     "no-nl-after-commit" = no extra NL after each commit
@@ -39,6 +39,7 @@ type VCS struct {
 	name         string           // Name of the VCS
 	subdirectory string           // Name of its metadata subdirectory
 	exporter     string           // Import/export style flags.
+	requires     stringSet        // Required tools
 	quieter      string           // How to make exporter quieter
 	styleflags   orderedStringSet // fast-export style flags
 	extensions   orderedStringSet // Format extension flags
@@ -58,6 +59,7 @@ type VCS struct {
 	notes        string           // Notes and caveats
 	// Hidden members
 	checkignore string // how to tell if directory is a checkout
+	idformat    string // ID display string format
 }
 
 // Constants needed in VCS class methods
@@ -101,6 +103,7 @@ func (vcs VCS) String() string {
 
 	return fmt.Sprintf("         Name: %s\n", vcs.name) +
 		fmt.Sprintf(" Subdirectory: %s\n", vcs.subdirectory) +
+		fmt.Sprintf("     Requires: %s\n", vcs.requires.String()) +
 		fmt.Sprintf("     Exporter: %s\n", vcs.exporter) +
 		fmt.Sprintf(" Export-Style: %s\n", vcs.styleflags.String()) +
 		fmt.Sprintf("   Extensions: %s\n", vcs.extensions.String()) +
@@ -138,7 +141,6 @@ func (vcs VCS) hasReference(comment []byte) bool {
 }
 
 var vcstypes []VCS
-var svntype *VCS
 
 // This one is special because it's used directly in the Subversion
 // dump parser, as well as in the VCS capability table.
@@ -167,6 +169,7 @@ func vcsInit() {
 			name:         "git",
 			subdirectory: ".git",
 			// Requires git 2.19.2 or later for --show-original-ids
+			requires:     newStringSet("git"),
 			exporter:     "git fast-export --show-original-ids --signed-tags=verbatim --tag-of-filtered-object=drop --use-done-feature --all",
 			quieter:      "",
 			styleflags:   newOrderedStringSet(),
@@ -176,7 +179,7 @@ func vcsInit() {
 			checkout:     "git checkout",
 			pathlister:   "git ls-files",
 			taglister:    "git tag -l",
-			branchlister: "git branch -q --list 2>&1 | cut -c 3- | egrep -v 'detached|^master$' || exit 0",
+			branchlister: "git branch -q --list 2>&1 | cut -c 3- | grep -E -v 'detached|^master$' || exit 0",
 			prenuke:      newOrderedStringSet(".git/config", ".git/hooks"),
 			preserve:     newOrderedStringSet(".git/config", ".git/hooks"),
 			authormap:    ".git/cvs-authors",
@@ -189,6 +192,7 @@ func vcsInit() {
 		{
 			name:         "bzr",
 			subdirectory: ".bzr",
+			requires:     newStringSet("bzr"),
 			exporter:     "bzr fast-export --no-plain .",
 			quieter:      "",
 			styleflags: newOrderedStringSet(
@@ -223,12 +227,56 @@ __pycache__
 bzr-orphans
 # Simulated bzr default ignores end here
 `,
-			cookies: reMake(tokenNumeric),
-			notes:   "Requires the bzr-fast-import plugin.",
+			cookies:  reMake(tokenNumeric),
+			notes:    "Requires the bzr-fast-import plugin.",
+			idformat: "%s",
+		},
+		{
+			name:         "brz",
+			subdirectory: ".brz",
+			requires:     newStringSet("brz"),
+			exporter:     "brz fast-export --no-plain .",
+			quieter:      "",
+			styleflags: newOrderedStringSet(
+				"export-progress",
+				"no-nl-after-commit",
+				"nl-after-comment"),
+			extensions: newOrderedStringSet(
+				"empty-directories",
+				"multiple-authors", "commit-properties"),
+			initializer:  "",
+			pathlister:   "",
+			taglister:    "brz tags",
+			branchlister: "brz branches | cut -c 3-",
+			importer:     "brz fast-import -",
+			checkout:     "brz checkout",
+			prenuke:      newOrderedStringSet(".brz/plugins"),
+			preserve:     newOrderedStringSet(),
+			authormap:    "",
+			project:      "https://www.breezy-vcs.org/",
+			ignorename:   ".brzignore",
+			dfltignores: `
+ # A simulation of brz default ignores, generated by reposurgeon.
+ *.a
+ *.o
+ *.py[co]
+ *.so
+ *.sw[nop]
+ *~
+ .#*
+ [#]*#
+ __pycache__
+ brz-orphans
+ # Simulated brz default ignores end here
+ `,
+			cookies:  reMake(tokenNumeric),
+			notes:    "Breezy capability is not well tested.",
+			idformat: "%s",
 		},
 		{
 			name:         "hg",
 			subdirectory: ".hg",
+			requires:     newStringSet("hg"),
 			exporter:     "",
 			styleflags: newOrderedStringSet(
 				"import-defaults",
@@ -253,11 +301,13 @@ bzr-orphans
 If there is no branch named 'master' in a repo when it is read, the hg 'default'
 branch is renamed to 'master'.
 `,
+			idformat: "%s",
 		},
 		{
 			// Styleflags may need tweaking for round-tripping
 			name:         "darcs",
 			subdirectory: "_darcs",
+			requires:     newStringSet("darcs"),
 			exporter:     "darcs fastconvert export",
 			quieter:      "",
 			styleflags:   newOrderedStringSet(),
@@ -372,13 +422,41 @@ core
 .DS_Store
 # Simulated darcs default ignores end here
 `,
-			cookies: reMake(),
-			project: "http://darcs.net/",
-			notes:   "Assumes no boringfile preference has been set.",
+			cookies:  reMake(),
+			project:  "http://darcs.net/",
+			notes:    "Assumes no boringfile preference has been set.",
+			idformat: "%s",
 		},
+		/*
+			{
+				name:         "pijul",
+				subdirectory: ".pijul",
+				requires:     newStringSet("pijul"),
+				exporter:     "",
+				quieter:      "",
+				styleflags:   newOrderedStringSet(),
+				extensions:   newOrderedStringSet(),
+				initializer:  "pijul init",
+				pathlister:   "pijul ls", // Undocumented
+				taglister:    "",
+				branchlister: "",
+				importer:     "",
+				checkout:     "",
+				prenuke:      newOrderedStringSet(),
+				preserve:     newOrderedStringSet(),
+				authormap:    "",
+				ignorename:   ".ignore",
+				dfltignores:  ``,
+				cookies:      reMake(),
+				project:      "http://pijul.org/",
+				notes:        "No importer/exporter pair yet.",
+				idformat:     "%s",
+			},
+		*/
 		{
 			name:         "mtn",
 			subdirectory: "_MTN",
+			requires:     newStringSet("mtn"),
 			exporter:     "mtn git_export",
 			quieter:      "",
 			styleflags:   newOrderedStringSet(),
@@ -431,13 +509,15 @@ _darcs
 *.bzr
 *.hg
 `,
-			cookies: reMake(),
-			project: "http://www.monotone.ca/",
-			notes:   "Exporter is buggy, occasionally emitting negative timestamps.",
+			cookies:  reMake(),
+			project:  "http://www.monotone.ca/",
+			notes:    "Exporter is buggy, occasionally emitting negative timestamps.",
+			idformat: "%s",
 		},
 		{
 			name:         "svn",
 			subdirectory: "locks",
+			requires:     newStringSet("svn"),
 			exporter:     "svnadmin dump  .",
 			quieter:      "--quiet",
 			styleflags:   newOrderedStringSet("import-defaults", "export-progress"),
@@ -457,10 +537,12 @@ _darcs
 			project:      "http://subversion.apache.org/",
 			notes:        "Run from the repository, not a checkout directory.",
 			checkignore:  ".svn",
+			idformat:     "r%s",
 		},
 		{
 			name:         "cvs",
 			subdirectory: "CVSROOT", // Can't be Attic, that doesn't always exist.
+			requires:     newStringSet("cvs-fast-export"),
 			exporter:     "find . -name '*,v' -print | cvs-fast-export --reposurgeon",
 			quieter:      "",
 			styleflags:   newOrderedStringSet("import-defaults", "export-progress"),
@@ -512,10 +594,33 @@ core
 			project:     "http://www.catb.org/~esr/cvs-fast-export",
 			notes:       "Requires cvs-fast-export.",
 			checkignore: "CVS",
+			idformat:    "%s",
+		},
+		{
+			name:         "sccs",
+			subdirectory: "SCCS",
+			requires:     newStringSet("sccs2rcs", "cvs-fast-export"),
+			exporter:     "sccs2rcs && find RCS -name '*,v' -print | cvs-fast-export --reposurgeon && rm -fr RCS",
+			quieter:      "",
+			styleflags:   newOrderedStringSet("export-progress"),
+			extensions:   newOrderedStringSet(),
+			initializer:  "",
+			importer:     "",
+			checkout:     "",
+			pathlister:   "",
+			preserve:     newOrderedStringSet(),
+			authormap:    "",
+			ignorename:   "",
+			dfltignores:  "", // Has none
+			cookies:      reMake(dottedNumeric),
+			project:      "http://www.catb.org/~esr/cvs-fast-export http://www.catb.org/~esr/sccs2rcs",
+			notes:        "Requires cvs-fast-export and sccs2rcs.",
+			idformat:     "%s",
 		},
 		{
 			name:         "rcs",
 			subdirectory: "RCS",
+			requires:     newStringSet("cvs-fast-export"),
 			exporter:     "find . -name '*,v' -print | cvs-fast-export --reposurgeon",
 			quieter:      "",
 			styleflags:   newOrderedStringSet("export-progress"),
@@ -531,15 +636,17 @@ core
 			cookies:      reMake(dottedNumeric),
 			project:      "http://www.catb.org/~esr/cvs-fast-export",
 			notes:        "Requires cvs-fast-export.",
+			idformat:     "%s",
 		},
 		{
 			name:         "src",
 			subdirectory: ".src",
+			requires:     newStringSet("src"),
 			exporter:     "src fast-export",
 			quieter:      "",
 			styleflags:   newOrderedStringSet(),
 			extensions:   newOrderedStringSet(),
-			initializer:  "src init",
+			initializer:  "",
 			importer:     "",
 			checkout:     "",
 			pathlister:   "src ls",
@@ -551,11 +658,13 @@ core
 			cookies:      reMake(tokenNumeric),
 			project:      "http://catb.org/~esr/src",
 			notes:        "",
+			idformat:     "%s",
 		},
 		{
 			// Styleflags may need tweaking for round-tripping
 			name:         "bk",
 			subdirectory: ".bk",
+			requires:     newStringSet("bk"),
 			exporter:     "bk fast-export --no-bk-keys",
 			quieter:      "-q",
 			styleflags:   newOrderedStringSet(),
@@ -574,11 +683,10 @@ core
 			cookies:      reMake(dottedNumeric), // Same as SCCS/CVS
 			project:      "https://www.bitkeeper.com/",
 			// No tag support, and a tendency to core-dump
-			notes: "Bitkeeper's importer is flaky and incomplete as of 7.3.1ce.",
+			notes:    "Bitkeeper's importer is flaky and incomplete as of 7.3.1ce.",
+			idformat: "%s",
 		},
 	}
-
-	svntype = &vcstypes[5]
 }
 
 // Import and export filter methods for VCSes that use magic files rather
